xquery version "3.1";

(:~
 : This module controls the serialization of any documents of the KMG.
 : Documents are given over by the TextGrid connector, prepared here, and given
 : over to the necessary functions for transformation/storage.
 :
 : @author Simon Sendler
 : @version 1.0
 : @since 1.0
 :)
module namespace kmg-serial="http://mollenhauer-edition.de/ns/kmg-serial";

import module namespace kmg-register-transformation="https://mollenhauer-edition.de/ns/register-transformation" at "kmg-register-transformation.xqm";
import module namespace bdn-utils="http://bdn-edition.de/ns/utils" at "utils.xqm";
import module namespace xml-utils="http://bdn-edition.de/portal/xml-utils" at "xml-utils.xqm ";
import module namespace bdn-odd="http://bdn-edition.de/ns/odd" at "schema.xqm";
import module namespace config="https://sade.textgrid.de/ns/config" at "/db/apps/sade/config.xqm";
import module namespace transformKMG="http://mollenhauer-edition.de/ns/transform";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare default element namespace "http://www.w3.org/1999/xhtml";

(:~
 : This function assures that any directories necessary to the serialization are
 : available.
 :)
declare function kmg-serial:directoryHelper() {
    let $resources := bdn-utils:assure-dir-available("resources")
    let $edition := bdn-utils:assure-dir-available("edition")
    let $edition := bdn-utils:assure-dir-available("lido")
    let $register := bdn-utils:assure-dir-available("register")
    let $aux := bdn-utils:assure-dir-available("aux")
    return
        true()
};

(:~
 : This function injects the Series Statement, which is stored in a seperate
 : document in TextGrid and referenced in the edition, into edition documents.
 : This change is stored, and not done in memory during transformation, as
 : complete XML-documents should be offered for download.
 :
 : @param $doc A TEI-XML document
 : @param $filename The filename, used in logging
 : @return $doc A TEI-XML document with the full Series Statement
 :)
declare function kmg-serial:inject-seriesStmt($doc as node()*, $filename as xs:string) {
    try {
            update replace $doc//tei:teiHeader//tei:seriesStmt
            with fn:doc($config:aux || "3qq81.xml")//*,
            util:log-system-out(util:system-time() || " " || $filename || ": Injection of seriesStmt successful."),
            $doc
        }
        catch * {
            util:log-system-out(util:system-time() || " " || $filename || ": Injection of seriesStmt failed. Some information might not be displayed correctly."),
                $doc
        }
};

(:~
 : Transforms a given XML file to a HTML file for the single view for each version.
 :
 : @author Michelle Weidling
 : @author Max-Ferdinand Zeterberg
 : @author Simon Sendler
 : @see
 : @version 1.0
 : @param $file The XML file to be transformed
 : @param $filename
 : @return xs:string The path where the HTML is stored
 : :)
declare function kmg-serial:html-single($file as node()*,
$filename as xs:string*) as xs:string*
{
    let $log := util:log-system-out(util:system-time() || " " || $filename || ":Start creating single views.")
    let  $wits := $file//tei:witness/@xml:id

    return
        for $wit in $wits
            let $log := util:log-system-out(util:system-time() || " " || $filename || ": Start creating single view for witness " || $wit || ".")
            let $content :=
                try {
                    element div {
                        transformKMG:transform($file, $filename, $wit)
                    }}
                catch * {
                    util:log-system-out(util:system-time() || " " || $filename || ": Could not create single view for witness " || $wit || ".")
                }
            let $new-filename := substring-before($filename, ".") || "-" || $wit ||".html"
            return
                try {
                    xmldb:store($config:edition, $new-filename, $content),
                    util:log-system-out(util:system-time() || " " || $filename || ": Finished single view for witness " || $wit || ".")}
                catch * {
                    util:log-system-out(util:system-time() || " " || $filename || ": Could not store single view.")
                }
};

(:~
 : Transforms a given file to a HTML file for the digital view.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $file The current file in intermediate format
 : @param $filename
 : @return xs:string The path where the HTML is stored
 : :)
declare function kmg-serial:html-digital($file as node()*,
$filename as xs:string*) as xs:string*
{
    let $log := util:log-system-out(util:system-time() || " " || $filename || ": Start creating digital view.")

    let $content :=
        try {
            transformKMG:transform($file, $filename, "digital")}
        catch * {
            util:log-system-out(util:system-time() || " " || $filename || ": Could not create digital view.")
        }
    let $rdgs-and-quotes := xml-utils:find-rdgs-and-quotes($content)
    let $svg-paths :=
        element div
        {
            $content//path
        }
    let $content :=
        element div{
            xml-utils:find-non-rdgs-and-non-quotes($content)
        }

    let $wits := distinct-values($rdgs-and-quotes/@wits[string-length(.) = 1])
    let $wits-panel := xml-utils:make-variants-overview($wits)
    let $variants-and-quotes :=
        element div
        {
            $wits-panel,
            $rdgs-and-quotes
        }

    let $new-filename := substring-before($filename, ".") || "-digital.html"
    let $new-filename-variants := substring-before($filename, ".") || "-digital-variants-and-quotes.html"
    let $new-filename-paths := substring-before($filename, ".") || "-digital-svg-paths.html"

    return
        try {
            xmldb:store($config:edition, $new-filename, $content),
            xmldb:store($config:edition, $new-filename-paths, $svg-paths),
            xmldb:store($config:edition, $new-filename-variants, $variants-and-quotes),
            util:log-system-out(util:system-time() || " " || $filename || ": Finished creating digital view.")}
        catch * {
            util:log-system-out(util:system-time() || " " || $filename || ": Could not create digital view.")
        }
};

declare function kmg-serial:html-commentary($file as node()*,
$filename as xs:string*) as xs:string*
{
    let $log := util:log-system-out(util:system-time() || " " || $filename || ":Start creating textual commentary.")
    let $content :=
        try {
            element div {
                transformKMG:transform($file, $filename, "commentary")
            }
        }
        catch * {
            util:log-system-out(util:system-time() || " " || $filename || ": Could not transform textual commentary.")
        }
    let $new-filename := substring-before($filename, ".") || "-commentary.html"

    return
        try {
            xmldb:store($config:edition, $new-filename, $content),
            util:log-system-out(util:system-time() || " " || $filename || ": Finished creating textual commentary.")
        }
        catch * {
            util:log-system-out(util:system-time() || " " || $filename || ": Could not create textual commentary.")
        }
};

(:~
 : This function coordinates the serialization of edition documents and hands
 : the document over to any necessary functions.
 :
 : @param $doc A TEI-XML document
 :)
declare function kmg-serial:serialize-edition($doc) {
    let $filename := util:document-name($doc)
    let $log := util:log-system-out(util:system-time() || " " || $filename || ": Document upload complete. Commencing serialization.")
    let $doc := kmg-serial:inject-seriesStmt($doc, $filename)

    let $single := kmg-serial:html-single($doc, $filename)
    let $digital :=
        if ( count($doc//tei:witness/@xml:id) gt 1 ) then
            kmg-serial:html-digital($doc, $filename)
        else
            util:log-system-out(util:system-time() || " " || $filename || ": Single witness text. Skipping digital view.")
    let $commentary := kmg-serial:html-commentary($doc, $filename)
    return
        util:log-system-out(util:system-time() || " " || $filename || ": Serialization complete.")
};

(:~
 : This function handles the storage of LIDO-XML.
 :
 : @param $uri The TextGrid-URI of the document
 :)
declare function kmg-serial:serialize-lido($uri) {
    xmldb:move($config:sade-data, $config:lido, $uri)
};

(:~
 : This function handles the storage of the registers.
 :
 :)
declare function kmg-serial:serialize-register() {
    let $write_persons_register := xmldb:store($config:register, "persons.xml", kmg-register-transformation:return-register($config:sade-data, "persons"))
    let $write_artworks_register := xmldb:store($config:register, "artworks.xml", kmg-register-transformation:return-register($config:sade-data, "artworks"))
        return
            true()
};


(:~
 : This function stores auxiliary files, such as the shared seriesStmt, in the
 : folder resources/aux
 :
 : @param $uri The document's TextGrid-URI
 :)
declare function kmg-serial:serialize-aux($uri as xs:string) {
    xmldb:move($config:sade-data, $config:aux, $uri)
};

(:~
 : This function receives documents from the TextGrid connector, identifies them
 : and hands them over to their respective functions/modules for further
 : processing.
 :
 : @param $uri A TextGrid-URI of an XML document
 :)
declare function kmg-serial:serialize($uri) {
    let $prepare := kmg-serial:directoryHelper()
    let $register := kmg-serial:serialize-register()
    let $doc := doc($config:sade-data || $uri)

    return
        if ( $uri = "3r8zv.xml" ) then
            (bdn-odd:main(),
            kmg-serial:serialize-aux($uri))
        else if ( $uri = "3qq81.xml" ) then
            kmg-serial:serialize-aux($uri)
        else if ( name($doc/*) = "lido:lido" ) then
            kmg-serial:serialize-lido($uri)
        else
            kmg-serial:serialize-edition($doc)
};