(:~
 : This module offers a variety of useful auxiliary functions that are necessary
 : for displaying the edition properly. It also provides methods that several
 : pages might use and that are not inherently of use for rendering the text of
 : the edition (these common function can be found in commons.xqm).
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @author Stefan Hynek
 :)

xquery version "3.1";

module namespace bdn-utils="http://bdn-edition.de/ns/utils";

declare namespace bdn="http://www.bdn-edition.de/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace textgrid="http://textgrid.info/namespaces/metadata/core/2010";
declare namespace tgmd="http://textgrid.info/namespaces/metadata/core/2010";

import module namespace edition="http://bdn-edition.de/ns/edition" at "edition.xqm";
import module namespace functx="http://www.functx.com";
import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace templates="http://exist-db.org/xquery/templates";

import module namespace console="http://exist-db.org/xquery/console";

declare option exist:serialize "expand-xincludes=no";

(:~
 : The sitemap function takes a general XML file in which all relevant URL sets
 : are listed and a matching XSL file in order to generate the sitemap which is
 : displayed at the "Projects" and "Works" pages (including all subpages).
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @param
 : @param
 : @return elemen(div) a div container with the sitemap panel in it
 :)
declare function bdn-utils:sitemap($node as node(), $model as map(*)) as element(div)
{
  let $doc := doc($config:app-root || "/sitemap.xml")
  let $xsl := doc($config:app-root || "/templates/xsl/sitemap.xsl")
  return
    element{"div"}
    {
      attribute{"class"}{"col-md-4"},
      element{"div"}
      {
        attribute{"class"}{"panel panel-blue"},
        element{"div"}
        {
          attribute{"class"}{"panel-heading"},
          element{"i"}
          {
            attribute{"class"}{"fas fa-map-signs"}
          },
          string(" Sitemap")
        },
        element{"div"}
        {
          attribute{"class"}{"panel-body"},
          transform:transform($doc, $xsl, ())
        }
      }
    }
};

(:~
 : Generates a sitemap without the surrounding div[@class = "col-md-4"] which is
 : necessary if there other panels above or below the sitemap panel.
 :
 : @author Michelle Weidling
 : @param
 : @param
 : @return element(div) the sitemap panel
 :)
declare function bdn-utils:reduced-sitemap($node as node(), $model as map(*))
as element(div)
{
  let $sitemap := bdn-utils:sitemap($node, $model)
  return $sitemap/div
};



(:~
 : Returns the author of a given file. File names are in the format of e.g.
 : "noesselt_1_2_3".
 :
 : @author Michelle Weidling
 : @param $file the filename
 : @return the author's name
 :)

 (: DEPRECATED? :)
declare function bdn-utils:get-author($file as xs:string) as xs:string
{
  substring-before($file, '_')
};


(:~
 : Returns the author belonging to a file from the whitelist.xml
 :
 : @author Michelle Weidling
 : @param $file the filename, e.g.22292
 : @return the author's name
 :)
declare function bdn-utils:get-author-from-whitelist($file as xs:string)
as xs:string
{
  let $whitelist := doc($config:aux || "whitelist.xml")
  let $bdn-file := $whitelist//bdn:file[matches(., $file)]
  return
    $bdn-file/ancestor::bdn:work/bdn:author/string()
};



(:~
 : Returns if the current file belongs to a BdN work that has already been
 : published.
 :
 : @author Michelle Weidling
 : @param $file the filename
 : @return xs:boolean
 :)

 (: TBD: ENTFERNEN, wird ersetzt von check-published
  : mock-up-function, weil statisch, info darüber aber in templates/authors/$author.xml und in whitelist.xml
 :)
declare function bdn-utils:has-published-author($file as xs:string) as xs:boolean
{
  let $author := bdn-utils:get-author($file)
  return
    switch($author)
      case "noesselt" return true()
      default return false()
};

(:~
 : Returns true when the author given is marked as published
 : in whitelist.xml
 :
 : @author Stefan Hynek
 : @param $author the id-name of the author
 : @return xs:boolean
 :)
declare function bdn-utils:check-published($author as xs:string) as xs:boolean
{
  let $whitelist := doc($config:aux || "whitelist.xml")
  let $result :=
    for $work in $whitelist//bdn:work
    where $work/bdn:author = $author
    return
      if ($work/bdn:published = "yes")
      then
        true()
      else
        false()

  return
    if (fn:empty($result))
    then
        false()
    else
        $result
};

(:~
 : Creates as xhtml:div element that contains the title of a selected edition
 : in the three edition views. A complete title consists of information about
 : the edition, the author's name and the current view.
 :
 : @author Michelle Weidling
 : @param $node the HTML node with the attribute which triggered this call
 : @param $model a map containing arbitrary data - used to pass information between template calls
 : @return xhtml:div
 :)
declare
  %templates:wrap
  function bdn-utils:title($node as node(), $model as map(*)) as element(div)
  {
    let $file := if (contains($model("file"), "noesselt"))
      then
        "noesselt_full.xml"
      else
        $model("file")
    let $doc := doc($config:sade-data || $file)
    let $edition := $model("edition")
    let $modeledition := request:get-parameter("model", "false")

    (: the full bibliographic reference of the edition :)
    let $title := $doc//tei:titleStmt//tei:title[@level = 'a']//text()
    let $author := $doc//tei:author/text()

    let $edt :=
      switch($edition)
        case "app"
          return
            "Textkritische Ansicht"
        case "digital"
          return
            "Interaktive textkritische Ansicht"
        case "foreword"
          return
            "Editorisches Vorwort"
        default
          return
            concat("Auflage ", $edition)

    let $header-type := if ($modeledition = "true")
      then "intern-header"
      else "public-header"

    return
      <div class="edition-header {$header-type}">
        <span class="author">{$author}</span>
        <span class="title">{$title}</span>
        <span class="edition" style="font-size: 12pt; font-weight: 300;">{$edt}</span>
      </div>
  };


(:~
 : Injects the apt CSS stylesheet to page_index.html depending on the file that
 : a user has selected in the edition view.
 :
 : @author Michelle Weidling
 : @param $node the HTML node with the attribute which triggered this call
 : @param $model a map containing arbitrary data - used to pass information between template calls
 : @return xhtml:link
 :)
declare function bdn-utils:css-injection($node as node(), $model as map(*))
as element(link)*
{
  let $file := request:get-parameter("file", "")
  let $mode := request:get-parameter("mode", "digital")
  return
    (
      element{"link"}{
        attribute{"rel"}{"stylesheet"},
        attribute{"href"}{"templates/css/neologie.css"}
      },
      if ($file = "2501b.2.xml")
        then
          element{"link"}
          {
            attribute{"rel"}{"stylesheet"},
            attribute{"href"}{"templates/css/modeledition.css"}
          }
        else
          (),
      if ($mode = "digital") then
          <link rel="stylesheet" href="templates/css/neologie-interactive.css" />
      else
          (),
      <link rel="stylesheet" href="templates/css/code-highlighting.css" />
    )
};

(:~
 : Creates a span that displays the current author and his work's title. This
 : xhtml:span is placed in the edition's navigation bar.
 :
 : @author Michelle Weidling
 : @param $node
 : @param $map
 : @return a span that displays the current author and his work's title
 :)
declare function bdn-utils:get-title($node as node(), $model as map(*))
as element(span)
{
  let $filename := request:get-parameter("file", "")
  let $author :=
      if (matches($filename, "-")) then
          bdn-utils:get-author-from-whitelist($filename)
        else
            ""
    return
        if ($author != "") then
          let $doc := doc($config:full-xmls || $author || "-full.xml")
          let $title := $doc//tei:title[@level = "a"]/tei:title[@type = "main"]/string()
          let $title := substring-before($title, "(")

          let $title-row-1 := substring-before($title, ",")
          let $title-row-2 := substring-after($title, ",")

          return
            <span>
              <span style="font-weight: bolder">{$title-row-1}</span>
              <br />
              {$title-row-2}
            </span>
    else
        <span />
};


(:~
 : Injects the grid layout depending on which view a user has selected. For the
 : interactive view we need a split layout with the main text on the left hand
 : and the variants on the right hand side, while the critical and single edition
 : views only have one wider col for the text (with an offset col on each side
 : for proper centering).
 :
 : This function is called by book.html.
 :
 : @author Michelle Weidling
 : @param $node the HTML node with the attribute which triggered this call
 : @param $model a map containing arbitrary data - used to pass information between template calls
 : @return grid layout with two text cols if interactive view is selected, grid layout with one text col otherwise
 :)
declare function bdn-utils:inject-cols($node as node(), $model as map(*))
as element(div)*
{
  let $mode := $model("edition")
  let $file := request:get-parameter("file", "")

  return
    if ($mode = "digital") then
      (templates:process(<div id="digital-text" class="col-sm-7" data-template="edition:display" />, $model),
      <div class="modalArea" />,
      templates:process(<div id="variants" class="col-sm-5" data-template="edition:display-variants-and-quotes" />, $model))
    else
      templates:process(<div id="text" class="col-md-10" data-template="edition:display" />, $model)
};


(:~
 : Injects the SVG layer in the interactive view with all relevant paths.
 :
 : This function is called by book.html.
 :
 : @author Michelle Weidling
 : @param $node the HTML node with the attribute which triggered this call
 : @param $model a map containing arbitrary data - used to pass information between template calls
 : @return xhtml:svg if the current view is the interactive one
 :)
declare function bdn-utils:inject-svg($node as node(), $model as map(*))
as element(svg)?
{
  let $mode := request:get-parameter("edition", "digital")
  let $file := request:get-parameter("file", "")
  let $svg-file := doc($config:edition || $file || "-digital-svg-paths.html")
  return
    if ($mode = "digital") then
      <svg class="svg-layer">
        {$svg-file//xhtml:path}
      </svg>
    else
      ()
};

(:~
 : Injects an additional JS file to page_index.html in case the digital view is
 : used.
 :
 : @author Michelle Weidling
 : @param $node the HTML node with the attribute which triggered this call
 : @param $model a map containing arbitrary data - used to pass information between template calls
 : @return xhtml:script
 :)
declare function bdn-utils:js-injection($node as node(), $model as map(*))
as element(script)*
{
  if (request:get-parameter("edition", "") => matches("digital")) then
    <script src="templates/js/kmg-interactive.js" />
  else
    ()
};

(:~
 : Creates a log file for a given author at /db/apps/sade/logs/.
 :
 : @author Michelle Weidling
 : @param $author the author's name
 : @return the location where the log has been stored
 :)
declare function bdn-utils:create-log($author as xs:string) as xs:string
{
  let $assure-dir-available := bdn-utils:assure-dir-available("logs")
  let $log-name := $author || "-log.xml"
  return xmldb:store($config:logs, $log-name, <log />)
};


declare function bdn-utils:add-log-entry($log-file as xs:string,
$message as xs:string) as empty-sequence()
{
  let $entry := <LogEntry timestamp="{util:system-time()}">{$message}</LogEntry>
  return update insert $entry into doc($log-file)/*
};

(:~
 : Checks if a given directory is available and creates it (under certain
 : circumstances) if this isn't the case.
 :
 : @author Michelle Weidling
 : @param $directory-name The name of a directory
 : @return the path to the new collection if it has been created, otherwise the empty sequence
 :)
declare function bdn-utils:assure-dir-available($directory-name as xs:string)
as item()*
{
  switch($directory-name)
    case "resources" return
      if (xmldb:collection-available($config:resources)) then
        $config:resources
      else
        xmldb:create-collection($config:data-root, "resources")

    case "logs" return
      if (xmldb:collection-available($config:logs)) then
        $config:logs
      else
        xmldb:create-collection($config:data-root, "logs")
   
    case "edition" return
      if (xmldb:collection-available($config:edition)) then
        $config:edition
      else
        xmldb:create-collection($config:resources, "edition")

    case "indices" return
      if (xmldb:collection-available($config:indices)) then
        ()
      else
        xmldb:create-collection($config:resources, "indices")

    case "aux" return
      if (xmldb:collection-available($config:aux)) then
        ()
      else
        xmldb:create-collection($config:resources, "aux")

    case "lido" return
      if (xmldb:collection-available($config:lido)) then
        ()
      else
        xmldb:create-collection($config:resources, "lido")
        
    case "register" return
      if (xmldb:collection-available($config:register)) then
        ()
      else
        xmldb:create-collection($config:resources, "register")
    default return ()
};


declare function bdn-utils:assure-dir-available($author as xs:string,
$directory-name as xs:string)
{
  if (xmldb:collection-available($config:edition || $author || $directory-name)) then
    $config:edition || $author || $directory-name

  else if (xmldb:collection-available($config:edition || $author)) then
    xmldb:create-collection($config:edition || $author, $directory-name)

  else
      (xmldb:create-collection($config:edition, $author),
      xmldb:create-collection($config:edition || $author, $directory-name))
};


(:~
 : Returns all URIs that belong to a given author.
 :
 : @author Michelle Weidling
 : @param $author The author's name
 : @return A sequence of URIs belonging to the author's work
 :)
declare function bdn-utils:retrieve-author-specific-uris($author as xs:string)
as xs:string*
{
  for $resource in collection($config:edition || $author || "/xml/")
    let $uri := base-uri($resource) => functx:substring-after-last("/")
    return
      if (matches($uri, $author)
      and not(matches($uri, "full"))) then
        $uri
      else
        ()
};

(:~
 : Retrieves the structure file for a given BdN author.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $author The current author
 : @return The opened structure document
 :)
declare function bdn-utils:get-structure-file($author as xs:string) as node()?
{
  switch($author)
    case "noesselt"
      return
        doc($config:sade-data || "25011.xml")
    case "griesbach"
      return
        doc($config:sade-data || "252bh.xml")
    case "less"
      return
        doc($config:sade-data || "38bf6.xml")
    case "teller"
      return
        doc($config:sade-data || "2skh7.xml")
    case "bahrdt-semler" return
        doc($config:sade-data || "257w3.xml")
    default return
        error(QName("http://bdn-edition.de/ns/utils", "UTILS001"), "No structure file known for " || $author || ".")
};


declare function bdn-utils:get-xml-dir($author as xs:string) as xs:string
{
  let $assure-coll-available := bdn-utils:assure-dir-available($author, "xml")
  return $config:edition || $author || "/xml/"
};


(:~
 : Since the digital view of our editions requires a certain screen width to
 : function properly, this information is shown as an overlay if the user's
 : screen has less than 768px in width.
 :
 : @author Michelle Weidling
 : @param $node
 : @param $model
 : @return a xhtml:section element containing the information of the overlay
 :)
declare function bdn-utils:small-screen-injection($node as node()*, $model as
map(*)) as element(section)?
{
  let $edition := request:get-parameter("edition", "digital")
  return
      if ($edition = "digital") then
          <section id="small-screen-warning">
              <div class="alert alert-warning">
                  <div class="alert-text">
                      <h4>
                          <i class="fas fa-exclamation-triangle" />
                      Informationen zur Bildschirmbreite</h4>
                      <hr />
                      <p>Die interaktive textkritische Ansicht setzt eine
                      Bildschirmbreite von mind. <strong>768px</strong> voraus.</p>
                      <p>Bitte verwenden Sie einen entsprechend breiten Bildschirm,
                      um diese Ansicht in vollem Umfang nutzen zu können.</p>

                      <div class="warning-button">
                          <button class="btn btn-warning" type="submit" onclick="hideOverlay()">Verstanden</button>
                      </div>
                  </div>
              </div>
          </section>
      else
          ()
};
