
const elem = document.documentElement;
const urlParams = new URLSearchParams(window.location.search);
const tgId = urlParams.get('file');
const zoomInButton = document.getElementById('zoom-in');
const zoomOutButton = document.getElementById('zoom-out');
const fullscreenButton = document.getElementById('icon-fullscreen');
const edition = document.getElementById('edition');

/**
 * This function opens the navigation on small screens.
 *
 * @param event
 * @function
 */
const openMobileNav = function openMobileNav(event) {
  event.preventDefault();
  document.querySelector('body').classList.add('nav-overlay');
};

/**
 * This function closes the navigation on small screns.
 *
 * @param event
 * @function
 */
const closeMobileNav = function closeMobileNav(event) {
  event.preventDefault();
  document.querySelector('body').classList.remove('nav-overlay');
};

/**
 * Search button.
 */
$('.search').on('click', 'a', (event) => {
  event.preventDefault();
  const input = $(event.target).parents('.search').find('input');
  input.toggleClass('show');
  input.focus();
});

$('.search').on('blur', 'input', (event) => {
  const input = $(event.target);
  if (input.val() === '') {
    input.removeClass('show');
  }
});

/**
 * Open fullscreen
 *
 * @function
 */
const openFullscreen = function openFullscreen() {
  if (elem.requestFullscreen) {
    elem.requestFullscreen();
  } else if (elem.mozRequestFullScreen) {
    /* Firefox */
    elem.mozRequestFullScreen();
  } else if (elem.webkitRequestFullscreen) {
    /* Chrome, Safari and Opera */
    elem.webkitRequestFullscreen();
  } else if (elem.msRequestFullscreen) {
    /* IE/Edge */
    elem.msRequestFullscreen();
  }
};

/**
 *  Close fullscreen
 *
 * @function
 */
const closeFullscreen = function closeFullscreen() {
  if (document.exitFullscreen) {
    document.exitFullscreen();
  } else if (document.mozCancelFullScreen) {
    /* Firefox */
    document.mozCancelFullScreen();
  } else if (document.webkitExitFullscreen) {
    /* Chrome, Safari and Opera */
    document.webkitExitFullscreen();
  } else if (document.msExitFullscreen) {
    /* IE/Edge */
    document.msExitFullscreen();
  }
};

/**
 * This function triggers the appearence of the fullscreen button and the display
 * of header and footer depending on whether fullscreen is opened or closed.
 *
 * @function
 */
const triggerFullscreenButton = function triggerFullscreenButton() {
  const fullscreenIcon = document.getElementById('icon-fullscreen').getElementsByClassName('svg-inline--fa')[0];
  fullscreenIcon.classList.toggle('fa-expand');
  fullscreenIcon.classList.toggle('fa-compress');
  const footer = document.getElementById('footer');
  footer.style.display = footer.style.display === 'none' ? '' : 'none';
  const navbar = document.getElementById('header');
  navbar.style.display = navbar.style.display === 'none' ? '' : 'none';
};

/**
 * This function determines whether fullscreen is opened by checking the
 * appearance of the fullscreen button.
 *
 * @function
 */
const fullscreen = function fullscreen() {
  const fullscreenIcon = document.getElementById('icon-fullscreen').getElementsByClassName('svg-inline--fa')[0];
  this.blur();
  if (fullscreenIcon.classList.contains('fa-expand')) {
    openFullscreen();
  } else {
    closeFullscreen();
  }
};

/** This function is used for the event handling of clicking the fullscreen
 * button.
 *
 * @function
 */
const exitHandler = function exitHandler() {
  if (document.webkitIsFullScreen === false) {
    triggerFullscreenButton();
  } else if (document.mozFullScreen === false) {
    triggerFullscreenButton();
  } else if (document.msFullscreenElement === false) {
    triggerFullscreenButton();
  } else if (document.webkitIsFullScreen === true) {
    triggerFullscreenButton();
  } else if (document.mozFullScreen === true) {
    triggerFullscreenButton();
  } else if (document.msFullscreenElement === true) {
    triggerFullscreenButton();
  }
};

if (document.addEventListener) {
  document.addEventListener('webkitfullscreenchange', exitHandler, false);
  document.addEventListener('mozfullscreenchange', exitHandler, false);
  document.addEventListener('fullscreenchange', exitHandler, false);
  document.addEventListener('MSFullscreenChange', exitHandler, false);
}

/**
 * This function retrieves the value of the font size from local storage.
 *
 * @see saveCurrentFontSize
 * @function
 */
const getCurrentFontSize = function getCurrentFontSize() {
  const currentFontSize = $('#edition').length === 0 ? localStorage.getItem('fontSizeSurround')
    : localStorage.getItem('fontSizeEdition');
  return currentFontSize;
};

/**
 * This function saves the desired value of the font size to local storage.
 * Other functions changing the font size retrieve this value.
 *
 * @param {*} input The value of the font size
 * @function
 */
const saveCurrentFontSize = function saveCurrentFontSize(input) {
  if ($('#edition').length === 0) {
    localStorage.setItem('fontSizeSurround', input);
  } else {
    localStorage.setItem('fontSizeEdition', input);
  }
};

/**
 * This function changes the font size as specified by the function changeFont
 *
 * @see changeFont
 * @function
 */
const setFontSize = function setFontSize() {
  let fontSize = getCurrentFontSize();
  fontSize += '%';
  if ($('#edition').length > 0) {
    $('#edition').css('font-size', fontSize);
  }
};

/**
  * This function prepares a change of font size, calling the functions to save
  * the desired value and to implement the actual change
  *
  * @param {int} change The value by which the font size is to be modified
  * @function
  */
const changeFont = function changeFont(change) {
  let currentFontSize = getCurrentFontSize();
  if ($('#edition').length === 0) {
    currentFontSize = localStorage.getItem('fontSizeSurround');
  } else {
    currentFontSize = localStorage.getItem('fontSizeEdition');
  }

  if (currentFontSize == null) {
    currentFontSize = '100';
  }

  const newFontSize = (parseInt(currentFontSize, 10) + change).toString();
  saveCurrentFontSize(newFontSize);
  setFontSize();
};

/**
 * This function initializes an increase in font size. It is called by the event
 * handler of the corresponding button.
 *
 * @function
 */
const increaseFont = function increaseFont() {
  this.blur();
  changeFont(10);
};

/**
 * This function initializes a decrease in font size. It is called by the event
 * handler of the corresponding button.
 *
 * @function
 */
const decreaseFont = function decreaseFont() {
  this.blur();
  changeFont(-10);
};

/**
 * This function checks whether an entry in KMG LIDO data is null or starts with
 * "#", indicating a placeholder.
 *
 * @param {string} input A string to check
 * @function
 */
const placeholderCheck = function placeholderCheck(input) {
  const newLocal = input == null;
  return newLocal ? true : !!input.startsWith('#');
};

/**
 * This function takes an URL and returns only the hostname, used for the
 * construction of links from URLs.
 *
 * @param {string} url An URL
 * @returns {string} The hostname of the URL provided
 * @function
 */
const getHost = function getHost(url) {
  return new URL(url).hostname.replace('www.', '');
};

/**
 * This function hides single entries in LIDO modals.
 *
 * @param target A HTML node
 * @function
 */
const lidoHide = function lidoHide(target) {
  target.parentNode.classList.add('hidden');
};

/**
 * This function takes a string containing an URL and returns a 'html:a' with
 * the hostname of the URL and a link icon as text and the URL as '@href'.
 *
 * @param {string} target An URL
 * @param {string} content The intended link text. If empty, defaults to hostname of target
 * @function
 */
const buildLink = function buildLink(target, content) {
  const newElement = document.createElement('a');
  let text = content || getHost(target);
  text += " <i class='fas fa-external-link-alt'/>";
  newElement.innerHTML = text;
  newElement.href = target;
  return newElement;
};

/**
 * This function appends a 'html:a' to an entry in the LIDO data modal.
 * If content is a placeholder or null, the list row will be hidden.
 *
 * @param {string} className The class name of the intended parent element
 * @param {string} target An URL
 * @param {string} content The intenden link text. Optional.
 * @function
 */
const makeLidoLink = function makeLidoLink(className, target, content) {
  const modal = document.getElementById('artModal');
  const elemnt = modal.querySelector(className);
  if ((typeof (content) === 'string' && placeholderCheck(content)) || placeholderCheck(target)) {
    lidoHide(elemnt);
  } else {
    const link = buildLink(target, content);
    elemnt.appendChild(link);
  }
};

/**
 * This function inserts LIDO-data into the modal.
 * If content is a placeholder or null, the list row will be hidden.
 *
 * @param {string} className The class name of the target element
 * @param {string} content The intended content
 * @function
 */
const makeLidoEntry = function makeLidoEntry(className, content) {
  const modal = document.getElementById('artModal');
  const elemnt = modal.querySelector(className);
  if (placeholderCheck(content) || (content === '')) {
    lidoHide(elemnt);
  }
  else {
    elemnt.innerHTML = content;
  }
};

/**
 * Helper function for LIDO-data that needs iteration.
 *
 * @param {string} className The class name of the target element in the modal
 * @param {string} object The object to be parsed
 * @function
 */
const iterateObjects = function iterateObjects(className, object) {
  const modal = document.getElementById('artModal');
  const elemnt = modal.querySelector(className);
  if (object === null) {
    lidoHide(elemnt);
  } else {
    object.forEach((obj) => makeLidoLink(className, obj, ''));
  }
};

/**
 * This function prepares the content of a modal displaying LIDO information on
 * works of art.
 *
 * @param {string} id A TG-URI referring to an object containing KMG LIDO data
 * @function
 */
const setArt = function setArtModalContent(id) {
  const modal = document.getElementById('artModal');
  const loading = modal.getElementsByClassName('loading')[0];
  const error = modal.getElementsByClassName('error')[0];
  const content = modal.getElementsByClassName('content')[0];
  loading.style.display = 'block';
  content.style.display = 'none';
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/api/lido/${id}`);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText);
      modal.querySelectorAll('.hidden').forEach((n) => n.classList.remove('hidden'));
      modal.querySelectorAll("[class^='Modal__']").forEach((n) => n.innerHTML = '');
      makeLidoEntry('.Modal__title', object.title);
      makeLidoEntry('.Modal__year', object.year);
      makeLidoEntry('.Modal__origin', object.origin);
      makeLidoEntry('.Modal__measurements', object.measurements);
      makeLidoEntry('.Modal__technique', object.technique);
      makeLidoEntry('.Modal__description', object.description);
      object.artist.href ? makeLidoLink('.Modal__artist', object.artist.href, object.artist.name)
        : makeLidoEntry('.Modal__artist', object.artist.name);
      const repositoryTitle = object.repository.name + ' (' + object.repository.location + ')';
      object.repository.href ? makeLidoLink('.Modal__repository', object.repository.href, repositoryTitle)
        : makeLidoEntry('.Modal__repository', repositoryTitle);
      typeof (object.reproductions) === 'string' ? makeLidoLink('.Modal__reproduction', object.reproductions,)
        : iterateObjects('.Modal__reproduction', object.reproductions);
      typeof (object.external) === 'string' ? makeLidoLink('.Modal__external', object.external,)
        : iterateObjects('.Modal__external', object.external);
      loading.style.display = 'none';
      content.style.display = 'block';
    } else if (this.status !== 200) {
      loading.style.display = 'none';
      error.style.display = 'block';
    }
  };
  xhttp.send();
};

/**
 * This function prepares the content of a modal displaying information on
 * organizations.
 *
 * @param {string} A GND-id referring an organization
 * @function
 */
const setOrg = function setOrgModalContent(id) {
  const modal = document.getElementById('orgModal');
  const loading = modal.getElementsByClassName('loading')[0];
  const error = modal.getElementsByClassName('error')[0];
  const content = modal.getElementsByClassName('content')[0];
  const source = modal.querySelector('.Modal__source').firstElementChild;
  source.innerHTML = '';
  loading.style.display = 'block';
  content.style.display = 'none';
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `https://lobid.org/gnd/${id[0]}.json`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText);

      modal.getElementsByClassName('Modal__name')[0].innerHTML = object.preferredName;
      id.forEach((e) => {
        source.append(
          buildLink(`http://d-nb.info/gnd/${e}`, `http://d-nb.info/gnd/${e}`)
        );
      });
      loading.style.display = 'none';
      content.style.display = 'block';
    } else if (this.status !== 200) {
      loading.style.display = 'none';
      error.style.display = 'block';
    }
  };
  xhttp.send();
};

/**
  * This function prepares the content of a modal displaying information on
  * periodicals.
  *
  * @param {string} id A GND-id referring a periodical
  * @function
  */
const setPeriodical = function setPeriodicalModalContent(id) {
  const modal = document.getElementById('periodicalModal');
  const loading = modal.getElementsByClassName('loading')[0];
  const error = modal.getElementsByClassName('error')[0];
  const content = modal.getElementsByClassName('content')[0];
  const source = modal.querySelector('.Modal__source').firstElementChild;
  source.innerHTML = '';
  loading.style.display = 'block';
  content.style.display = 'none';

  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `https://lobid.org/gnd/${id[0]}.json`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText);
      modal.getElementsByClassName('Modal__name')[0].innerHTML = object.preferredName;
      id.forEach((e) => {
        source.append(
          buildLink(`http://d-nb.info/gnd/${e}`, `http://d-nb.info/gnd/${e}`)
        );
      });
      loading.style.display = 'none';
      content.style.display = 'block';
    } else if (this.status !== 200) {
      loading.style.display = 'none';
      error.style.display = 'block';
    }
  };
  xhttp.send();
};

/**
 * This function prepares the content of a modal displaying information on brand
 * names.
 *
 * @todo Finish the function
 * @param {string} id A GND-id referring a brand name
 * @function
 */
const setBrandname = function setBrandnameModalContent(id) {
  const modal = document.getElementById('brandnameModal');
  const loading = modal.getElementsByClassName('loading')[0];
  const error = modal.getElementsByClassName('error')[0];
  const content = modal.getElementsByClassName('content')[0];
};

/**
 * This function prepares the content of a modal displaying biographical data.
 *
 * @param {string} id A GND-id referring a person
 * @function
 */
function setPersonModalContent(id) {
  const modal = document.getElementById('personModal');
  const loading = modal.getElementsByClassName('loading')[0];
  const error = modal.getElementsByClassName('error')[0];
  const content = modal.getElementsByClassName('content')[0];
  const source = modal.querySelector('.Modal__source').firstElementChild;
  source.innerHTML = '';
  modal.getElementsByClassName('personModal__birth')[0].parentNode.style.display = '';
  modal.getElementsByClassName('personModal__death')[0].parentNode.style.display = '';
  loading.style.display = 'block';
  content.style.display = 'none';

  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `https://ref.de.dariah.eu/pndsearch/json?id=${id[0]}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      let object = JSON.parse(this.responseText).entry;
      let dateOfBirth = object.dateOfBirth ? object.dateOfBirth.split('-').reverse().join('.') : null;
      let dateOfDeath = object.dateOfDeath ? object.dateOfDeath.split('-').reverse().join('.') : null;
      modal.getElementsByClassName('personModal__name')[0].innerHTML = object.preferredNameForThePerson;
      dateOfBirth  === null ? modal.getElementsByClassName('personModal__birth')[0].parentNode.style.display = 'none' :
        modal.getElementsByClassName('personModal__birth')[0].innerHTML = dateOfBirth;
      dateOfDeath === null ? modal.getElementsByClassName('personModal__death')[0].parentNode.style.display = 'none' :
        modal.getElementsByClassName('personModal__death')[0].innerHTML = dateOfDeath;
      id.forEach((e) => {
        source.append(
          buildLink(`http://d-nb.info/gnd/${e}`, `http://d-nb.info/gnd/${e}`)
        );
      });
      loading.style.display = 'none';
      content.style.display = 'block';
    } else if (this.status !== 200) {
      loading.style.display = 'none';
      error.style.display = 'block';
    }
  };
  xhttp.send();
}

/**
 * This function prepares the content of a modal displaying bibliographical data.
 *
 * @param {string} id An id of an object in the KMG Zotero-group
 * @param {string} mode A string indicating editorial or authorial bibliographical data
 * @function
 */
function setRefModalContent(id, mode) {
  const modal = document.getElementById('refModal');
  const loading = modal.getElementsByClassName('loading')[0];
  const error = modal.getElementsByClassName('error')[0];
  const content = modal.getElementsByClassName('content')[0];
  loading.style.display = 'block';
  content.style.display = 'none';
  if (mode === 'KMG') {
    modal.getElementsByClassName('modal-title--KMG')[0].style.display = 'block';
    modal.getElementsByClassName('modal-title--base')[0].style.display = 'none';
  } else {
    modal.getElementsByClassName('modal-title--KMG')[0].style.display = 'none';
    modal.getElementsByClassName('modal-title--base')[0].style.display = 'block';
  }
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/zotero/items/${id}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText)[id];

      modal.getElementsByClassName('content')[0].innerHTML = object.bib;
      loading.style.display = 'none';
      content.style.display = 'block';
    } else if (this.status !== 200) {
      loading.style.display = 'none';
      error.style.display = 'block';
    }
  };
  xhttp.send();
}

/**
 * This function prepares the content of a modal displaying data on musical works.
 *
 * @param {string} id A GND-id referring to a piece of music
 * @function
 */
const setMusic = function setMusicModalContent(id) {
  const modal = document.getElementById('musicModal');
  const loading = modal.getElementsByClassName('loading')[0];
  const error = modal.getElementsByClassName('error')[0];
  const content = modal.getElementsByClassName('content')[0];
  const source = modal.querySelector('.Modal__source').firstElementChild;
  source.innerHTML = '';
  loading.style.display = 'block';
  content.style.display = 'none';
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `https://lobid.org/gnd/${id[0]}.json`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText);
      modal.getElementsByClassName('Modal__name')[0].innerHTML = object.preferredName;
      modal.getElementsByClassName('Modal__author')[0].innerHTML = object.librettist[0].label;
      modal.getElementsByClassName('Modal__date')[0].innerHTML = object.dateOfPublication[0];
      id.forEach((e) => {
        source.append(
          buildLink(`http://d-nb.info/gnd/${e}`, `http://d-nb.info/gnd/${e}`)
        );
      });
      loading.style.display = 'none';
      content.style.display = 'block';
    }
    else if (this.status !== 200) {
      loading.style.display = 'none';
      error.style.display = 'block';
    }
  };
  xhttp.send();
};

/**
 * This function allows for the movement of modals by dragging the header.
 *
 * @param {object} elmnt An element with the class "modal-content"
 * @function
 */
const dragElement = function dragElement(elmnt) {
  let pos1 = 0; let pos2 = 0; let pos3 = 0; let pos4 = 0;
  function elementDrag(e) {
    e = e || window.event;
    e.preventDefault();
    pos1 = pos3 - e.clientX;
    pos2 = pos4 - e.clientY;
    pos3 = e.clientX;
    pos4 = e.clientY;
    elmnt.style.top = `${elmnt.offsetTop - pos2}px`;
    elmnt.style.left = `${elmnt.offsetLeft - pos1}px`;
  }
  function closeDragElement() {
    document.removeEventListener('mouseup', closeDragElement);
    document.removeEventListener('mousemove', elementDrag);
  }
  function dragMouseDown(e) {
    e = e || window.event;
    e.preventDefault();
    pos3 = e.clientX;
    pos4 = e.clientY;
    document.addEventListener('mouseup', closeDragElement);
    document.addEventListener('mousemove', elementDrag);
  }
  if (elmnt.getElementsByClassName('modal-header')[0]) {
    elmnt.getElementsByClassName('modal-header')[0].addEventListener('mousedown', dragMouseDown);
  } else {
    elmnt.addEventListener('mousedown', dragMouseDown);
  }
};

/**
 * Allow modals to be dragged.
 */
const modals = document.querySelectorAll('.modal');
modals.forEach((modal) => dragElement(modal.querySelector('.modal-content')));

/**
 * This function inserts the metadata of the current edition into the info
 * modal.
 *
 * @function
 */
const getMetadata = function getMetadata() {
  const infoModalBody = document.getElementById('infoModal').querySelector('.modal-body');
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/api/meta/${tgId}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText);
      infoModalBody.querySelector('#editionMainTitle').innerHTML = object.title;
      infoModalBody.querySelector('#edition-dateCreated').innerHTML = object.created.string;
      infoModalBody.querySelector('#edition-dateModified').innerHTML = object.modified.string;
      infoModalBody.querySelector('#edition-dateTransformed').innerHTML = object.transformed.string;
      infoModalBody.querySelector('#edition-textGridURI').innerHTML = object.tguri;
    }
  };
  xhttp.send();
};

/**
 * This function hides any items in the info modal's legend if there are no
 * corresponding instances in the current edition.
 *
 * @function
 */
const checkUsedSymbols = function checkUsedSymbols() {
  const infoModalBody = document.getElementById('infoModal').querySelector('.modal-body');
  const legend = infoModalBody.querySelector('.js-edition-info-section--legend');
  const iconClasses = ['.artname', '.brandname', '.musicname', '.orgname', '.periodname', '.persname', '.correction', '.js-editorial-icon--bibl-KMG', '.js-editorial-icon--bibl', '.reference', '.js-editorial-icon--note'];
  iconClasses.forEach((iconClass) => (edition.querySelector(iconClass) ? '' : legend.querySelector(iconClass).style.display = 'none'));
};

/**
 * This function copies text referenced by the element's @data-clipboard-target to the
 * clipboard
 *
 * @function
 */
const copyToClipboard = function copyToClipboard() {
  const targetId = this.getAttribute('data-clipboard-target').replace('#', '');
  const targetText = document.getElementById(targetId).innerHTML;
  this.blur();
  navigator.clipboard.writeText(targetText);
};

/**
 * This function opens the correction modal.
 *
 * @param event
 * @function
 */
const openCorr = function openCorrModal(event) {
  event.stopPropagation();
  const { target } = this.dataset;
  $('#corrModal').modal('toggle');
  setCorr(target);
};

/**
 * This function opens the person modal.
 *
 * @param event
 * @function
 */
const openPerson = function openPersonModal(event) {
  event.stopPropagation();
  const target = this.dataset.target.split(' ');
  setPersonModalContent(target);
  $('#personModal').modal('show');
};


/**
 * This function opens the bibliographical reference modal.
 *
 * @param event
 * @function
 */
const openRef = function openRefModal(event) {
  event.stopPropagation();
  const { target } = this.dataset;
  const mode = this.classList.contains('js-editorial-icon--bibl-KMG') ? 'KMG' : '';
  $('#refModal').modal('show');
  setRefModalContent(target, mode);
};

/**
 * This function opens the editorial note modal.
 *
 * @param event
 * @function
 */
const openNote = function openNoteModal(event) {
  event.stopPropagation();
  const { target } = this.dataset;
  setNote(target);
  $('#noteModal').modal('show');
};

/**
 * This function opens the organization modal.
 *
 * @param event
 * @function
 */
const openOrg = function openOrgModal(event) {
  event.stopPropagation();
  const target = this.dataset.target.split(' ');
  $('#orgModal').modal('show');
  setOrg(target);
};

/**
 * This function opens the artwork modal.
 *
 * @param event
 * @function
 */
const openArt = function openArtModal(event) {
  event.stopPropagation();
  const {target} = this.dataset;
  $('#artModal').modal('show');
  setArt(target);
};

/**
 * This function opens the musical works modal.
 *
 * @param event
 * @function
 */
const openMusic = function openMusicModal(event) {
  event.stopPropagation();
  const target = this.dataset.target.split(' ');
  $('#musicModal').modal('show');
  setMusic(target);
};

/**
 * This function opens the modal displaying information on periodicals.
 *
 * @param event
 * @function
 */
const openPeriodical = function openperiodicalModal(event) {
  event.stopPropagation();
  const target = this.dataset.target.split(' ');
  $('#periodicalModal').modal('show');
  setPeriodical(target);
};

/**
 * This function opens the modal displaying information on brand names.
 *
 * @param event
 * @function
 */
function openbrandnameModal(event) {
  event.stopPropagation();
  const target = this.dataset.target.split(' ');
  $('#brandnameModal').modal('show');
  setBrandname(target);
}

/**
 * This function opens the info modal.
 *
 * @param event
 * @function
 */
const openInfoModal = function openInfoModal() {
  $('#infoModal').modal('toggle');
  getMetadata();
  checkUsedSymbols();
};

/**
 * add event listeners
 */
const infoButton = document.querySelector('.js-edition-menu__info');
const clipboardButtons = document.querySelectorAll('.clipboard-btn');
const mobileNavOpen = document.getElementById('header').querySelector('nav').querySelector('.bars');
const mobileNavClose = document.getElementById('header').querySelector('nav').querySelector('.cross');

mobileNavOpen?.addEventListener('click', openMobileNav);
mobileNavClose?.addEventListener('click', closeMobileNav);
infoButton?.addEventListener('click', openInfoModal);
zoomInButton?.addEventListener('click', increaseFont);
zoomOutButton?.addEventListener('click', decreaseFont);
fullscreenButton?.addEventListener('click', fullscreen);

const downloadButton = document.querySelector('.js-edition-menu__download');

/**
   * This function sets the @download and @href of the download button according to the current
   * edition.
   *
   * @function
   */
const setDownloadLink = function setDownloadLink() {
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/api/meta/${tgId}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState == 4 && this.status == 200) {
      const object = JSON.parse(this.responseText);
      downloadButton.download = `Mollenhauer:${object.title}_${tgId}.xml`;
    }
  };
  xhttp.send();
  downloadButton.href = `/exist/restxq/api/full-xml/${tgId}`;
};

downloadButton ? setDownloadLink() : '';
clipboardButtons.forEach((button) => button.addEventListener('click', copyToClipboard));

/**
   * This function assigns the necessary events to any icons in edition texts.
   *
   * @function
   */
const assignTextEvents = function assignTextEvents() {
  const corrIcons = document.querySelectorAll('.js-editorial-icon--corr');
  const personIcons = document.querySelectorAll('.js-editorial-icon--person');
  const biblKMGIcons = document.querySelectorAll('.js-editorial-icon--bibl-KMG');
  const biblIcons = document.querySelectorAll('.js-editorial-icon--bibl');
  const noteIcons = document.querySelectorAll('.js-editorial-icon--note');
  const orgIcons = document.querySelectorAll('.js-editorial-icon--org');
  const artIcons = document.querySelectorAll('.js-editorial-icon--art');
  const periodicalIcons = document.querySelectorAll('.js-editorial-icon--periodica');
  const musicIcons = document.querySelectorAll('.js-editorial-icon--music');

  corrIcons.forEach((e) => { e.addEventListener('click', openCorr); });
  personIcons.forEach((e) => { e.addEventListener('click', openPerson); });
  biblKMGIcons.forEach((e) => { e.addEventListener('click', openRef, false); });
  biblIcons.forEach((e) => { e.addEventListener('click', openRef); });
  noteIcons.forEach((e) => { e.addEventListener('click', openNote); });
  orgIcons.forEach((e) => { e.addEventListener('click', openOrg); });
  artIcons.forEach((e) => { e.addEventListener('click', openArt); });
  periodicalIcons.forEach((e) => { e.addEventListener('click', openPeriodical); });
  musicIcons.forEach((e) => { e.addEventListener('click', openMusic); });
};

/**
   * This function sets the content of a modal for editorial notes.
   *
   * @param {id} id A HTML-id referring to the content to be included in the modal body
   * @function
   */
const setCorr = function setCorrModalContent(id) {
  const modalContent = document.getElementById(id).innerHTML;
  const modalBody = document.getElementById('corrModal').getElementsByClassName('modal-body')[0];
  modalBody.innerHTML = modalContent;
  assignTextEvents();
};

/**
   * This function sets the content of a modal for editorial notes.
   *
   * @param {string} id A HTML-id referring to the content to be included in the modal body
   * @function
   */
const setNote = function setNoteModalContent(id) {
  const modalContent = document.getElementById(id).innerHTML;
  const modalBody = document.getElementById('noteModal').getElementsByClassName('modal-body')[0];
  modalBody.innerHTML = modalContent;
  assignTextEvents();
};

document.addEventListener('DOMContentLoaded', assignTextEvents);
